<?php

namespace App\Http\Livewire\Users;

use App\Models\User;
use Livewire\Component;
use App\Helpers\LogHelper;
use App\Models\JobLicense;
use App\Models\UserLicense;
use App\Models\JobDescription;
use App\Models\JobAuthorization;
use App\Models\UserAuthorization;
use Illuminate\Support\Facades\DB;
use Spatie\Permission\Models\Role;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Password;

class EditUser extends Component
{
    protected $listeners = ['showModal'];

    public $username = "";
    public $email;
    public $first_name = "";
    public $last_name = "";
    public $phone;
    public $password;
    public $password_confirmation;
    public $user_id;
    public $user;

    public $jobDescription;
    public $jobDescriptions = [];

    public $jobLicenses = [];
    public $selectedJobLicenses = [];
    public $roles = [];
    public $selectedRoles = [];
    public $jobAuthorizations = [];
    public $selectedJobAuthorizations = [];
    public $showModal = false;

    public function showModal(User $user)
    {
        $this->user = $user;
        $this->user_id = $user->id;
        $this->username = $user->username;
        $this->email = $user->email;
        $this->first_name = $user->userData->first_name;
        $this->last_name = $user->userData->last_name;
        $this->phone = $user->userData->phone;
        $this->jobDescription = $user->userData->job_description_id;
        $this->selectedJobLicenses = $user->licenses->pluck('name');
        $this->selectedJobAuthorizations = $user->authorizations->pluck('name');
        $this->selectedRoles = $user->roles->pluck('name');

        $this->showModal = true;
        $this->emit('refreshJobLicenseSelect');
        $this->emit('refreshRoleSelect');
        $this->emit('refreshJobAuthorizationSelect');
    }

    protected $rules = [
        'username' => 'required',
        'email' => 'required|email',
        'first_name' => 'required',
        'last_name' => 'required',
        'phone' => 'required',
        'jobDescription' => 'required',
        'selectedJobLicenses' => 'required',
        'selectedRoles' => 'required',
        'selectedJobAuthorizations' => 'required',
    ];

    public function mount()
    {
        $this->jobDescriptions = JobDescription::all();
        $this->jobLicenses = JobLicense::all();
        $this->roles = Role::all();
        $this->jobAuthorizations = JobAuthorization::all();
    }

    public function render()
    {
        return view('livewire.users.edit-user');
    }

    public function closeModal()
    {
        $this->resetInputFields();
        $this->resetErrorBag();
    }

    private function resetInputFields()
    {
        $this->jobDescription = [];
        $this->selectedJobLicenses = [];
        $this->selectedRoles = [];
        $this->selectedJobAuthorizations = [];
    }

    public function save()
    {
        $this->validate();
        DB::beginTransaction();
        try {
            $user = User::find($this->user_id);
            $user->update([
                'username' => $this->username,
                'email' => $this->email,
                'password' => Hash::make($this->email),
            ]);
            $user->save();

            $user->userData->update([
                'user_id' => $this->user->id,
                'first_name' => $this->first_name,
                'last_name' => $this->last_name,
                'initials' => mb_convert_case(mb_substr($this->first_name, 0, 1, 'UTF-8') . mb_substr($this->last_name, 0, 1, 'UTF-8'), MB_CASE_UPPER, "UTF-8"),
                'job_description_id' => $this->jobDescription,
                'phone' => $this->phone,
            ]);

            // $this->user->licenses()->sync($this->selectedJobLicenses);

            UserLicense::where('user_id', $user->id)->delete();
            foreach ($this->selectedJobLicenses as $license) {
                $user_license = JobLicense::where('name', $license)->first();
                UserLicense::create([
                    'user_id' => $user->id,
                    'job_license_id' => $user_license->id,
                ]);
            }

            $user->syncRoles($this->selectedRoles);

            // $this->user->authorizations()->sync($this->selectedJobAuthorizations);

            UserAuthorization::where('user_id', $user->id)->delete();
            foreach ($this->selectedJobAuthorizations as $authorization) {
                $user_authorization = JobAuthorization::where('name', $authorization)->first();
                UserAuthorization::create([
                    'user_id' => $user->id,
                    'job_authorization_id' => $user_authorization->id,
                ]);
            }

            DB::commit();

            $message = __('messages.User') . " $this->first_name $this->last_name " . __('messages.is Edited');
            $messageLog = "Корисникот $this->first_name $this->last_name е ажуриран";
            $data = ['user' => $user];
            LogHelper::logInfo($messageLog, $data);

            $this->reset();
            $this->resetErrorBag();

            redirect()->route('users.index')->with('message', $message);
        } catch (\Exception $e) {
            DB::rollback();
            $message = $e->getMessage();
            $data = ['error' => $e->getMessage()];
            LogHelper::logMessageError($message, $data);
            $this->dispatchBrowserEvent('notifyUser', [
                'style' => 'danger',
                'status' =>  __('messages.Error'),
                'message' => $e->getMessage()
            ]);
        }
    }

    public function resetPassword()
    {
        Password::sendResetLink(['email' => $this->user->email]);
    }
}
